# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-21 10:58:00 +0900 (Tue, 21 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'net/http'
unless Net::HTTP.instance_methods(true).member? "propfind"
  require 'vfs/scheme/http/davext'
end

module VFS;  module SchemeHandler;  module HTTP;
  module Common

    PROP_CACHE_TIME = 30

    def check_scheme(uri)
      unless uri.scheme == "http"
	raise Error::InvalidURI,
	  "Invalid HTTP URL '#{uri}' was speficied."
      end
    end

    def init_common_attr
      @capability   = []
      @prop_cache   = nil
      @prop_cache_t = 0
      @header       = nil
    end
    private :init_common_attr
    attr_writer :capability, :header
    attr_accessor :prop_cache, :prop_cache_t
    private :prop_cache, :prop_cache=, :prop_cache_t,
      :prop_cache_t=, :header=

    def capability?(method)
      capability.member? method      
    end

    def capability
      return @capability unless @capability.length == 0
      begin
        capability = http_request(:options) { |rep|
          rep["allow"].split(/\s*,\s*/)
        }
      rescue Net::HTTPServerException, Net::HTTPFatalError
        capability = ["GET"]
      end
    end

    def property(elem)
      unless capability? "PROPFIND"
	return nil
      end
      if prop_cache == nil || Time.now.to_i - prop_cache_t > PROP_CACHE_TIME
	log("Start fetching property.")
        begin
          http_request(:propfind, nil, {"Depth" => "0"}) { |rep|
            log("*BUG/NotInplementedYet* must unescape XML entity.", Log::WARNING)
            __send__ :prop_cache=, rep.body
          }
        rescue Net::HTTPServerException
          return nil
        end
	__send__ :prop_cache_t=, Time.now.to_i
	log("Property was fetched")
      end
      log("Scaning property '#{elem}'.", Log::DEBUG)
      m = Regexp.new("<#{elem}(?:\s+[^>]*)?>(.*?)</#{elem}>").match(prop_cache)
      if m && m[1]
	m[1].tr("\n\r\t", " ")
      else
	nil
      end
    end

    def header(name=nil)
      if @header == nil
	log("Start fetching header.")
	http_request(:head) { |rep|
	  __send__ :header=, rep.each_header{}
	}
	log("Header was fetched")
      end
      name == nil ? @header : @header[name]
    end

    def http_connect(t_uri)
      if ENV.member? "http_proxy"
	proxy = URI.parse(ENV["http_proxy"])
	proxy_host = proxy.host
	proxy_port = proxy.port
      end
      Net::HTTP.version_1_1
      log("Opening HTTP connection to #{uri.host}.", Log::INFO)

      Net::HTTP.new(t_uri.host, t_uri.port, proxy_host, proxy_port)
    end

    def http_request(method=:get, *rest_args, &block)
      # FIXME!
      #  * keep-alive support
      #  * Threaded?
      t_uri = uri
      begin
        thread_critical_bak = Thread.critical
        Thread.critical = false
	http_connect(t_uri).start { |http|
	  log("Requesting '#{method}' for '#{t_uri.path}' to server.", Log::DEBUG)
	  response = http.send(method, t_uri.path, *rest_args)
	  Array === response and response = response[0]
	  log("Data was fetched from web server. Go yield.", Log::DEBUG)
	  if block_given?
	    yield(response)
	  else
	    response
	  end
	}
      rescue Net::ProtoRetriableError => err
	log("Retriable error was occured in HTTP request. (detail: #{err})",
	    Log::INFO)
	t_uri = URI.parse(err.response['location'])
	retry
      ensure
        Thread.critical = thread_critical_bak
      end
    end # http_request

  end # Common
end; end; end
