module Net
  class HTTP
    class Proppatch < HTTPRequest
      METHOD = 'PROPPATCH'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Lock < HTTPRequest
      METHOD = 'LOCK'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Unlock < HTTPRequest
      METHOD = 'UNLOCK'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Options < HTTPRequest
      METHOD = 'OPTIONS'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = false
    end
    class Propfind < HTTPRequest
      METHOD = 'PROPFIND'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Delete < HTTPRequest
      METHOD = 'DELETE'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Move < HTTPRequest
      METHOD = 'MOVE'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Copy < HTTPRequest
      METHOD = 'COPY'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Mkcol < HTTPRequest
      METHOD = 'MKCOL'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
  end # HTTP
  
  class HTTP < Protocol
    def proppatch( path, body, initheader = nil ) 
      res = request(Proppatch.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def lock( path, body, initheader = nil ) 
      res = request(Lock.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def unlock( path, body, initheader = nil ) 
      res = request(Unlock.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def options( path, initheader = nil ) 
      res = request(Options.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def propfind( path, body = nil, initheader = { 'Depth' => '0' } )
      res = request(Propfind.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def delete( path, initheader = { 'Depth' => 'Infinity' } )
      res = request(Delete.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def move( path, initheader = nil )
      res = request(Move.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def copy( path, initheader = nil )
      res = request(Copy.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def mkcol( path, body = nil, initheader = nil)
      res = request(Mkcol.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
  end # HTTP < Protocol
  
  class HTTPGenericRequest
    SEND_BUFFER_SIZE = 10240
    remove_method :send_request_with_body
    def send_request_with_body( sock, ver, path, body )
      unless @header.has_key?('content-length')
        if body.respond_to?(:length) && body.length
          @header['content-length'] = body.length.to_s
        elsif body.respond_to?(:size) && body.size
          @header['content-length'] = body.size.to_s
        else
          $stderr.puts 'net/http: warning: cannot get Content-Length; request may be failed.'
        end
      end
      @header.delete 'transfer-encoding'
      
      unless @header['content-type']
        $stderr.puts 'net/http: warning: Content-Type did not set; using application/x-www-form-urlencoded' if $VERBOSE
        @header['content-type'] = 'application/x-www-form-urlencoded'
      end
      request sock, ver, path
      if body.kind_of?(IO)
        while ( ! body.eof? )
          sock.write body.read(SEND_BUFFER_SIZE)
        end
      else
        sock.write body
      end
    end
  end # HTTPGenericRequest
end # Net

