# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-22 08:13:22 +0900 (Wed, 22 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/zip/common'
require 'vfs/compat'

module VFS;  module SchemeHandler;
  module ZIP;
    class File < VFS::File
      include VFS::SchemeHandler::ZIP::Common

      def open_for_read(*args, &block)
        if directory?
          # FIXME. correct error class
          raise Errno::EISDIR
        end
        zipentry # force entry caching. Hmmm...
        VFS::CompatLayer.override
	stream = zipentry.get_input_stream
        VFS::CompatLayer.restore
	class << stream
	  def sysread(len, *buf)
	    t = read(len)
	    raise EOFError if t == nil
	    if buf.length > 0
	      raise Error::Unsupported,
		"sysread emulation: direct input to buf has not been supported."
	    end
	    t
	  end
	end
	if block_given?
	  begin
	    yield(stream)
	  ensure
	    stream.close
	  end
	else
	  stream
	end
      end

      def open_for_write(*args, &block)
        flag = conv_openflang_to_const(args[0])
        if flag & RDWR != 0
          raise Error::Unsupported,
                "RDWR open for zip has not been supported."
        end
        if flag & APPEND != 0
          raise Error::Unsupported,
                "append write mode for zip has not been supported."
        end
        ziparc_open(! flag & CREAT) { |zf|
          if (flag & CREAT) && !zf.find_entry(zipcont)
            # FIXME. correct error
            raise Errno::ENOENT
          end
          stream = zf.get_output_stream(zipcont)
          if block_given?
            begin
              yield(stream)
            ensure
              stream.close
            end
          else
            stream
          end
        }
      end

      def rename_internal(dest)
        new_arc, new_cont = parse_zipurl(dest)
        if new_arc == @ziparc
          ziparc_open { |zf|
            zf.file.rename(zipcont, new_cont)
          }
          @ziparc  = new_arc
          @zipcont = new_cont
          @uri = dest
          self
        else
          rename_external(dest)
        end
      end

      return_false :chardev?, :pipe?, :blockdev?, :socket?, :symlink?

      def utime(new_atime, new_mtime = nil)
        ziparc_open { |zf|
          zf.file.utime(new_atime, zipcont)
        }
        self
      end

      def delete
        ziparc_open { |zf|
          zf.file.delete(zipcont)
        }
        self
      end
      alias :unlink :delete

      method_def = ""
      [:zero?, :mtime, :exist?, :truncate, :link, :setgid?, :stat,
       :chmod, :size, :atime, :directory?, :setuid?, :pipe,
       :expand_path, :ftype, :readlink, :size?, :exists?, :ctime,
       :symlink, :lstat, :sticky?, :chown, :file?,].each { |m|
        method_def << %Q{
          def #{m}(*args)
            ziparc_open { |zf|
              zf.file.#{m}(zipcont, *args)
            }
          end
        }
      }
      class_eval(method_def)

      [:exist?, :readable?, :readable_real?, :writable?, :writable_real?,
       :owned?, :executable?, :executable_real?, :grpowned?].each{|m|
        alias_method m, :exists?
      }

    end # VFS::SchemeHandler::ZIP::File
end; end; end


if __FILE__ == $0
  VFS.loglevel = VFS::Log::DEBUG if $DEBUG
#  f = "zip:/home/sugi/data/mail/local/tosc-00000.zip/1"
#  f = "zip:/home/sugi/public_html/test.zip/dir1/file11"
  f = "zip:http://sugi.nemui.org/pub/incomming/test.zip/dir1/file11"
  vfs = VFS::SchemeHandler::ZIP::File.new(f)
  p vfs
  p VFS::scheme(f)
  p vfs.scheme
  p vfs.exists?
  p vfs.mtime
  p vfs.size
  p vfs.basename
  p vfs.dirname
  p vfs.scheme
  io = vfs.open
  puts io.read(40)
  io.close
  vfs.open { |io| puts io.read(40) }
  vfs.open("w") {|io|
    io << "Now: " << Time.now.to_s
  }
end
