# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-20 03:58:25 +0900 (Mon, 20 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/common'

module VFS
  class Tempbuf

    def initialize(url)
      @io         = nil
      @path       = url
      @is_closed  = false
      @close_hook = nil
      @binmode    = false
      begin
	require 'stringio'
	log("creating as StringIO.", Log::DEBUG)
	@io = StringIO.new
      rescue LoadError
	@@use_stringio = false
	switch_to_tempfile
      end
    end
    attr_reader   :path, :io
    attr_accessor :close_hook

   def binmode?
     @binmode
   end

   def binmode
     @binmode = true
     self
   end

    def kind_of?(c)
      if c == IO
        return true
      end
      super
    end

    def inspect
      "\#<#{self.class} buf=#{io} bind='#{path}'>"
    end

    def switch_to_tempfile
      require 'tempfile'
      log("Switching to Tempfile.", Log::INFO)
      new_io = Tempfile.new('ruby-vfs')
      binmode? && new_io.binmode
      if io && io.size > 0
	new_io << io.string
	new_io.seek(io.pos)
      end
      @io = new_io
    end

    def <<(str)
      io << str
      if @@use_stringio && StringIO === io && @@string_max < io.size
	log("Buffer size was exceeded in StringIO", Log::INFO)
	switch_to_tempfile
      end
      self
    end

    def real_path
      io.path
    end

    def close
      if close_hook
	log("call close hook.", Log::DEBUG)
	io.flush
	close_hook.call
      end
      @is_closed = true
      if @@use_stringio && StringIO === io
	io.rewind
      elsif Tempfile === io
	io.flush
	io.rewind
      else
	raise Error::Unknown, "unkown error! please bug report!!"
      end
      self
    end

    def open
      @is_closed = false
      self
    end

    def renew
      log("renew was called. dispose of old IO and all hooks.", Log::DEBUG)
      __send__ :close_hook=, nil
      begin
	io.close
      rescue
      end
      begin
	@io = StringIO.new
      rescue NameError
	switch_to_tempfile
      end
      self
    end

    def closed?
      @is_closed
    end

    def to_io
      self
    end

    def reopen(to, mode = "r")
      raise Error::Unsupported
#      fito.kind_of?(IO)
    end

    [:flush, :seek, :rewind].each { |m|
      class_eval %{
        def #{m}(*args, &block)
          io.send :#{m}, *args, &block
          self
        end
      }
    }

    (IO.instance_methods(false).delete_if{ |m|
      instance_methods(false).member?(m) ||
      m == "close_write" || m == "close_read"
     } + Enumerable.instance_methods(false) + [:size] ).each{|m|
      class_eval %{
        def #{m}(*args, &block)
          io.send :#{m}, *args, &block
        end
      }
    }
    
    def log(*args)
      args[0] = "VFS(TempBuf): #{args[0]}"
      Log.log(*args)
    end
    private :log

  end # Tempbuf

end # VFS
